import pandas as pd
import os
from itertools import combinations


class SearchContent:
    def __init__(self, import_file, site, filetype, sep, combine):
        filetypes = ['swf', 'pdf', 'ps', 'dwf', 'kml', 'kmz', 'gpx', 'hwp', 'htm', 'html',
                     'xls', 'xlsx', 'ppt', 'pptx', 'doc', 'docx', 'odp', 'ods', 'odt',
                     'rtf', 'svg', 'tex', 'txt', 'text', 'bas', 'c', 'cc', 'cpp', 'cxx',
                     'h', 'hpp', 'cs', 'java', 'pl', 'py', 'wml', 'wap', 'xml']
        self.import_file = import_file
        self.site = [] if site == '' else site.split(',')
        self.filetype = [] if filetype == '' else filetype.split(',')
        self.filetype = [i for i in self.filetype if i in filetypes]
        self.sep = sep
        self.combine = combine

    @classmethod
    def user_input(cls):
        while 1:
            import_file = input('请输入文件名')
            if os.path.exists(import_file) and import_file.split('.')[-1] in ['txt', 'csv', 'xlsx']:
                break
            else:
                if not os.path.exists(import_file):
                    print('！！！文件不存在！！！')
                if import_file.split('.')[-1] not in ['txt', 'csv', 'xlsx']:
                    print('！！！文件类型不符！！！')

        site = input('请输入指定站点，如有多个用逗号分隔，如无请直接回车')
        filetype = input('请输入指定文件类型，如有多个用逗号分隔，如无请直接回车')
        sep = '\n'
        combine = 1
        return cls(
            import_file,
            site,
            filetype,
            sep,
            combine
        )

    def convert_to_list(self):
        """
        :return: 待查询的字符串
        """
        if self.import_file.split('.')[-1] == 'txt':
            f = open(self.import_file, 'r', encoding='utf-8')
            keywords = f.read().strip().split(sep=self.sep)
        elif self.import_file.split('.')[-1] == 'csv':
            keywords = pd.read_csv(self.import_file, header=None).iloc[:, 0].tolist()
        elif self.import_file.split('.')[-1] == 'xlsx':
            keywords = pd.read_excel(self.import_file, header=None).iloc[:, 0].tolist()
        else:
            print('请选择正确格式文件')
            return None

        keywords = [k.strip() for k in keywords if k.strip() != '']

        if self.combine > 1:
            for i in range(2, self.combine+1):
                cl = list(map(lambda x: ' '.join(x), combinations(keywords, i)))
                keywords.extend(cl)
        if len(self.site) > 0:
            keywords = ['{} site:{}'.format(string, site_) for string in keywords for site_ in self.site]
        if len(self.filetype) > 0:
            keywords = ['{} filetype:{}'.format(string, file_) for string in keywords for file_ in self.filetype]
        return keywords



from xsearch.page import SearchPage, IndexPage
from xsearch.input import SearchContent
from xsearch.output import CrawledContent


def search():
    search_content = SearchContent.user_input()
    search_list = search_content.convert_to_list()
    c_content = CrawledContent.user_input()
    search_page = SearchPage.user_input()
    n = 30
    for i in range(0, len(search_list), n):
        search_page.startup()
        s_list = search_list[i:i+n]
        search_page.submit_keyword(s_list[0])
        index_page = IndexPage(search_page.driver)

        for kw in s_list:
            index_page.submit_keyword(kw)
            res = index_page.get_search_result()
            c_content.update(kw, res)
            if c_content.analyse:
                c_content.extract_tags()
            c_content.export()
        index_page.driver.quit()
    print("--------查询结束--------")
#
#
# if __name__ == '__main__':
#     search()



import os
import json
import time
import re
import random
from bs4 import BeautifulSoup
from selenium import webdriver
from urllib.parse import urlparse
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as ec


class SearchPage:

    def __init__(self, domain):
        self.domain = re.sub(r"http://|https://|www.", "", domain)
        # self.search_box_file = os.getcwd().replace('\\', '/') + "/param/searchbox_of_searchpage.json"
        self.search_box_file = "param/searchbox_of_searchpage.json"
        with open(self.search_box_file, encoding='utf-8') as json_file:
            self.search_box_dict = json.load(json_file)
        self.search_method = self.search_box_dict.get(self.domain, {}).get('method')
        self.search_name = self.search_box_dict.get(self.domain, {}).get('name')
        self.driver = None

    @classmethod
    def user_input(cls):
        domains = ['google.com', 'google.com.hk', 'baidu.com', 'bing.com', 'sogou.com', 'weixin.sogou.com',
                   '36kr.com', 'huxiu.com', 'iyiou.com', 'qbitai.com', 'pingwest.com', 'awtmt.com',
                   'kejilie.com', 'leiphone.com', 'csdn.net', '51cto.com', 'woshipm.com', 'qianzhan.com']
        while 1:
            domain = input("请输入搜索域名")
            if domain in domains:
                break
            else:
                print('暂不支持，请重新输入')
        return cls(domain)

    def startup(self):
        chrome_options = Options()
        chrome_options.add_argument("--disable-blink-features=AutomationControlled")
        self.driver = webdriver.Chrome(options=chrome_options)
        self.driver.get('https://' + self.domain)

    @staticmethod
    def get_search_element(driver, method, name):
        """给定网页浏览器和查找方法，返回搜索框对象
        :param driver: selenium.webdriver.chrome.webdriver.WebDriver
        :param method: str 支持name/class/id/placeholder
        :param name: str
        :return:
        """
        if method == 'name':
            element = driver.find_element_by_name(name)
        elif method == 'class':
            element = driver.find_element_by_class_name(name)
        elif method == 'id':
            element = driver.find_element_by_id(name)
        elif method == 'placeholder':
            element = driver.find_element(By.XPATH, "//input[@placeholder='{}']".format(name))
        else:
            element = None
        return element

    def submit_keyword(self, keyword):
        search_box = self.get_search_element(self.driver, self.search_method, self.search_name)
        search_box.clear()
        search_box.send_keys(keyword)
        time.sleep(0.5)
        search_box.send_keys(Keys.ENTER)


class IndexPage:

    def __init__(self, driver):
        self.driver = driver
        self.driver.implicitly_wait(10)
        self.domain = urlparse(self.driver.current_url).netloc.replace('www.', '')

        self.search_box_file = os.getcwd().replace('\\', '/') + '/param/searchbox_of_indexpage.json'
        with open(self.search_box_file, encoding='utf-8') as file1:
            self.search_box_dict = json.load(file1)
        self.search_method = self.search_box_dict.get(self.domain, {}).get('method')
        self.search_name = self.search_box_dict.get(self.domain, {}).get('name')

        self.search_result_file = os.getcwd().replace('\\', '/') + '/param/search_result.json'
        with open(self.search_result_file, encoding='utf-8') as file2:
            self.search_result_dict = json.load(file2)
        self.result_tag = self.search_result_dict.get(self.domain, {}).get('tag')
        self.result_class = self.search_result_dict.get(self.domain, {}).get('class_name')

    @staticmethod
    def get_search_element(driver, method, name):
        """给定网页浏览器、查找方法、查找内容，返回搜索框对象
        :param driver: selenium.webdriver.chrome.webdriver.WebDriver
        :param method: str 支持name/class/id/placeholder
        :param name: str
        :return:
        """
        if method == 'name':
            element = driver.find_element_by_name(name)
        elif method == 'class':
            element = driver.find_element_by_class_name(name)
        elif method == 'id':
            element = driver.find_element_by_id(name)
        elif method == 'placeholder':
            element = driver.find_element(By.XPATH, "//input[@placeholder='{}']".format(name))
        else:
            element = None
        return element

    def submit_keyword(self, keyword):
        search_box = self.get_search_element(self.driver, self.search_method, self.search_name)
        search_box.clear()
        search_box.send_keys(keyword)
        search_box.send_keys(Keys.ENTER)
        time.sleep(1)

    def get_search_result(self):

        def convert_wechat_link(link):
            b = int(random.random() * 100) + 1
            a = link.find("url=")
            converted_link = 'https://weixin.sogou.com' + link + "&k=" + str(b) + "&h=" + link[a + 25 + b]
            return converted_link

        res_list = []
        if self.domain == 'google.com' or self.domain == 'google.com.hk':
            content = self.driver.page_source.encode('utf-8')
            soup = BeautifulSoup(content, 'lxml')
            ele_list = soup.find_all('div', 'g')
            for ele in ele_list:
                try:
                    res_list.append([ele.find('h3').text, ele.find('a').get('href'), ele.find('span', 'aCOpRe').text])
                except AttributeError:
                    continue

        elif self.domain == 'baidu.com':
            element = WebDriverWait(self.driver, 10).\
                until(ec.presence_of_element_located((By.CLASS_NAME, "search_tool")))
            content = self.driver.page_source.encode('utf-8')
            soup = BeautifulSoup(content, 'lxml')
            ele_list = soup.find_all('div', 'result c-container new-pmd')
            for ele in ele_list:
                try:
                    res_list.append([ele.find('h3').text, ele.find('a').get('href'),
                                     ele.find('div', 'c-abstract').text])
                except AttributeError:
                    continue

        elif self.domain == 'bing.com':
            content = self.driver.page_source.encode('utf-8')
            soup = BeautifulSoup(content, 'lxml')
            ele_list = soup.find_all('li', 'b_algo')
            for ele in ele_list:
                try:
                    res_list.append([ele.find('h2').text, ele.find('a').get('href'), ele.find('p').text])
                except AttributeError:
                    continue

        elif self.domain == 'sogou.com':
            content = self.driver.page_source.encode('utf-8')
            soup = BeautifulSoup(content, 'lxml')
            ele_list = soup.find_all('div', 'vrwrap')
            for ele in ele_list:
                try:
                    res_list.append([ele.find('h3').text, ele.find('a').get('href'),
                                     ele.find('div', 'strBox').text])
                except AttributeError:
                    continue

        elif self.domain == 'weixin.sogou.com':
            content = self.driver.page_source.encode('utf-8')
            soup = BeautifulSoup(content, 'lxml')
            ele_list = soup.find('ul', 'news-list').find_all('li')
            for ele in ele_list:
                try:
                    res_list.append([ele.find('h3').text.strip(), convert_wechat_link(ele.find('a').get('href')),
                                     ele.find('p', 'txt-info').text])
                except AttributeError:
                    continue

        else:
            ele_list = self.driver.find_elements_by_class_name(self.result_class)
            for ele in ele_list:
                try:
                    res_list.append([ele.find_element_by_tag_name('a').get_attribute('href'), ele.text])
                except NoSuchElementException:
                    continue

        return res_list



import os
import json
import csv
import jieba.analyse


class CrawledContent:

    def __init__(self, analyse, export_file):
        self.analyse = analyse
        self.export_file = export_file
        self.next = {}
        self.result = {}

    @classmethod
    def user_input(cls):
        while 1:
            try:
                analyse = int(input('需要文本分析提取关键词则键入1，否则键入0'))
                break
            except ValueError:
                print("请重新输入")
        while 1:
            export_file = input('请输入导出文件，支持csv/json')
            if export_file.split('.')[-1] in ['csv', 'json']:
                break
        return cls(
            analyse,
            export_file
        )

    def update(self, keyword, result_list):
        self.next = {keyword: result_list}
        self.result.update(self.next)

    def export(self):
        if self.export_file.split('.')[-1] == 'csv':
            with open(self.export_file, 'a', newline='', encoding='utf-8-sig') as csv_file:
                writer = csv.writer(csv_file)
                key = list(self.next.keys())[0]
                val = self.next[key]
                for i in val:
                    writer.writerow([key, val.index(i) + 1] + i)

        if self.export_file.split('.')[-1] == 'json':
            if os.path.exists(self.export_file):
                with open(self.export_file, encoding='utf-8') as json_file:
                    result_dict = json.load(json_file)
                result_dict.update(self.next)
            else:
                result_dict = self.next
            with open(self.export_file, 'w', encoding='utf-8') as json_file:
                json.dump(result_dict, json_file, ensure_ascii=False, indent=4)

    def extract_tags(self, idx=-1, topK=5, allowPOS=('n', 'nr', 'ns')):
        key = list(self.next.keys())[0]
        val = self.next[key]
        for k in range(len(val)):
            try:
                tags = jieba.analyse.extract_tags(val[k][idx], topK=topK, allowPOS=allowPOS)
                self.next[key][k].append(' '.join(tags))
            except AttributeError:
                pass
